# -*- coding: utf-8 -*-
"""Wing logics."""
import copy


def xwing(s, p, b, boxl, mirror, verbose):
    """X-wing."""
    from kaidoku.misc import (cell)
    message = ''
    double = {}
    for i in range(9, 27):
        for j in range(len(mirror[i][0])):
            if len(mirror[i][1][j]) == 2:
                n = mirror[i][0][j]
                if n not in double:
                    double[n] = []
                a = boxl[i][mirror[i][1][j][0]]
                b = boxl[i][mirror[i][1][j][1]]
                if i < 18:  # row
                    c = (b - a) * 9 + a % 9
                else:  # column
                    c = 81 + b - a + a // 9
                m = n * 162 + c
                if m in double:
                    double[m].append([a, b])
                else:
                    double[m] = [[a, b]]
    wing = {}
    for m in double:
        if len(double[m]) > 1:
            n = m // 162
            if len(double[m]) > 2:
                message = 'This sudoku has no solution because '
                + cell(double[m][0][0]) + ' or ' + cell(double[m][0][1])
                + ' is ' + str(n) + ' and ' + cell(double[m][1][0]) + ' or '
                + cell(
                    double[m][1][1]) + ' is ' + str(n) + ' and '
                + cell(double[m][2][0]) + ' or ' + cell(double[m][2][1])
                + ' is ' + str(n) + ', but all of them cannot be true.'
                return (s, p, message, False, True)
            if n in wing:
                wing[n].append([double[m][0][0], double[m][0][1],
                                double[m][1][0], double[m][1][1]])
            else:
                wing[n] = [([double[m][0][0], double[m][0][1],
                             double[m][1][0], double[m][1][1]])]
    for n in wing:
        win = copy.copy(wing[n])
        for square in win:
            if [square[0], square[2], square[1], square[3]] in wing[n]:
                wing[n].remove(square)
                wing[n].remove([square[0], square[2], square[1], square[3]])
        if len(wing[n]) > 0:
            if verbose > 1:
                if verbose == 2:
                    message = 'X-wing can be found.'
                else:
                    message = 'X-wing is found. ' + cell(wing[n][0][0]) + \
                        ', ' + cell(wing[n][0][1]) + ', ' + cell(
                        wing[n][0][2]) + ' and ' + cell(wing[n][0][3]) + \
                        ' are X-wing of ' + str(n) + '. ' + str(n) + \
                        ' is removed from'
            w = wing[n][0]
            i = [0, 0]
            if w[1] - w[0] < 9:  # row
                i[0] = w[0] % 9 + 18
                i[1] = w[1] % 9 + 18
            else:  # column
                i[0] = w[0] // 9 + 9
                i[1] = w[1] // 9 + 9
            for j in [0, 1]:
                box = boxl[i[j]]
                position = mirror[i[j]][1][mirror[i[j]][0].index(n)]
                for pos in position:
                    i2 = box[pos]
                    if i2 not in w:
                        p[i2][n - 1] = 0
                        if sum(p[i2]) == 1:
                            s[i2] = p[i2].index(1) + 1
                        if verbose > 2:
                            message += ' ' + cell(i2)
            if verbose > 2:
                message += '.'
            return (s, p, message, True, False)
    return (s, p, message, False, False)


def xywing(s, p, b, pair, verbose):
    """XY-wing."""
    from kaidoku.chain import (findchain)
    from kaidoku.misc import (cell)
    message = ''
    if len(pair) < 3:
        return (s, p, message, False, False)
    for i in pair:
        chainx = findchain(pair, [i[0], i[1]], b)
        chainy = findchain(pair, [i[0], i[2]], b)
        if len(chainx) > 1 and len(chainy) > 1:
            for x in chainx:
                for y in chainy:
                    # if x[1] == y[1] and x[0] not in b[y[0]]:
                    if x[1] == y[1]:
                        for bb in b[x[0]]:
                            if bb in b[y[0]]:
                                if s[bb] == 0 and p[bb][x[1] - 1] == 1 and i not in p[bb]:
                                    if verbose > 1:
                                        if verbose == 2:
                                            message = 'XY-wing can be found.'
                                        else:
                                            message = 'XY-wing of ' + \
                                                cell(
                                                    i[0]) + ' (' + str(i[1]) + ',' + \
                                                str(i[2]) + '), '
                                            message += cell(x[0]) + ' (' + \
                                                str(i[1]) + ',' + \
                                                str(x[1]) + '), ' + \
                                                cell(y[0]) + ' (' + str(i[2]) + \
                                                ',' + str(y[1]) + ') '
                                            message += 'removes ' + \
                                                str(x[1]) + ' from ' + \
                                                cell(bb) + '.'
                                    p[bb][x[1] - 1] = 0
                                    if sum(p[bb]) == 1:
                                        s[bb] = p[bb].index(1) + 1
                                        if verbose > 2:
                                            message += '\nTherefore ' + \
                                                cell(bb) + ' = ' + \
                                                str(s[bb]) + '.'
                                    return (s, p, message, True, False)

    return (s, p, message, False, False)



"""Trial and search."""
import copy
import datetime
from kaidoku.misc import blank
from kaidoku.misc import cell


def trial(s, p, maxstep, verbose, b, pair):
    """Trial."""
    if len(pair) == 0:
        return (s, p, 0, '', False, False)
    for x in pair:
        i = x[0]
        for j in x[1:]:
            s2 = copy.deepcopy(s)
            p2 = copy.deepcopy(p)
            s2[i] = j
            p2[i] = [10]
            found = True
            minblank = blank(s2) - maxstep
            if minblank < 0:
                minblank = 0
            message2 = ''
            line = 1
            while found and blank(s2) > minblank:
                s2, p2, message3, found, err = scan(s2, p2, 4, b)
                message2 += '(' + str(line) + ') ' + message3 + '\n'
                line += 1
            if err:
                message = ''
                if verbose > 1:
                    if verbose == 2:
                        message = 'Determine ' + cell(i) + ' by trial.'
                    else:
                        message = 'Trial. If ' + cell(i) + ' is ' + str(j) + \
                            ',\n' + message2.replace(
                            'This sudoku has no solution because', 'Contradiction:')
                p[i][j - 1] = 0
                s[i] = p[i].index(1) + 1
                step = message2.count('=')
                if verbose > 2:
                    message += 'Therefore ' + \
                        cell(i) + ' is ' + str(s[i]) + '.'
                return (s, p, step, message, True, False)
    return (s, p, 0, '', False, False)


def scan(s, p, verbose, b):
    """Scan."""
    from kaidoku.calc import hidsing
    from kaidoku.calc import naksing
    from kaidoku.misc import check
    s, p, message, found, err = naksing(s, p, b, verbose)  # Naked single
    if found or err:
        return (s, p, message, found, err)
    message, err = check(s)
    if err:
        return (s, p, message, False, True)
    s, p, message, found, err = hidsing(s, p, verbose)  # Hidden single
    return (s, p, message, found, err)


def search(s, p, verbose, depth, maxdepth, endtime, b, pb):
    """Full search."""
    from kaidoku.calc import (solveone)

    if datetime.datetime.now() > endtime:
        message = ''
        if verbose > 0:
            message = 'Search time limit'
        return (s, p, message, 9 + depth, depth, False, False)

    message = ''
    smallest = 10
    pair = []
    pairnum = [[], [], [], [], [], [], [], [], []]
    for j in range(81):
        if s[j] == 0:
            if sum(p[j]) == 2:
                pair.append(j)
                for k in range(9):
                    if p[j][k] == 1:
                        pairnum[k].append([j])
            if sum(p[j]) < smallest:
                i = j
                smallest = sum(p[j])
    if len(pair) > 1:
        m = 0
        for j in range(9):
            if len(pairnum[j]) > m:
                i = int(pairnum[j][0][0])

    solution = []
    removed = []

    for n in range(9):
        if p[i][n] == 0:
            continue
        s2 = copy.deepcopy(s)
        p2 = copy.deepcopy(p)
        s2[i] = n + 1
        while (True):
            s2, p2, message, logic, depth2, found, err = solveone(
                s2, p2, 0, depth, maxdepth, endtime, b, pb)
            if found:
                if err:  # Multiple solutions
                    if depth2 > depth:
                        depth = depth2
                    return (s2, p2, message, logic, depth, True, True)
                else:
                    if blank(s2) == 0:
                        solution.append([n, s2])
                        break
            else:
                if err:  # No solution
                    if depth2 > depth:
                        depth = depth2
                    removed.append(n)
                    break
                else:
                    break
        if len(solution) > 1:
            break

    if depth == 1:
        logic = 'Search'
    else:
        logic = 'Deep search'

    if len(solution) > 1:
        if verbose > 1:
            message = 'Search with depth ' + \
                str(depth) + ' shows that this sudoku has multiple solutions.'
        s = [solution[0][1], solution[1][1]]
        return (s, p, message, logic, depth, True, True)
    if len(removed) > 0:
        for n in removed:
            p[i][n] = 0
        if sum(p[i]) == 0:
            if verbose > 1:
                message = 'Search with depth ' + \
                    str(depth) + ' shows that ' + \
                    cell(i) + ' has no candidate.'
            return (s, p, message, logic, depth, False, True)
        if sum(p[i]) == 1:
            s[i] = p[i].index(1) + 1
            if verbose > 1:
                message = 'Search with depth ' + \
                    str(depth) + ' shows that ' + \
                    cell(i) + ' should be ' + str(s[i])
            return (s, p, message, logic, depth, True, False)
        if verbose > 1:
            message = 'Search with depth ' + \
                str(depth) + ' shows that ' + \
                cell(i) + ' is not ' + str(removed)
        return (s, p, message, logic, depth, True, False)
    else:
        if verbose > 1:
            message = 'Search with depth ' + \
                str(depth) + ' on ' + cell(i) + ' did not give any clue.'
        return (s, p, message, logic, depth, False, False)



# -*- coding: utf-8 -*-
"""Miscellaneous modules."""
import copy


def conv(problem):
    """Convert problem to array."""
    s = []
    if problem.find(',') == -1:
        for i in problem:
            if i.isdigit():
                s = s + [int(i)]
            else:
                s = s + [0]
        if len(s) != 81:
            return ('Error in input.')
    else:
        for line in problem.split(","):
            if (len(line)) == 9:
                for i in line:
                    if i.isdigit():
                        s = s + [int(i)]
                    else:
                        s = s + [0]
            else:
                return ('Error in input: ' + line, True)

    if len(s) == 81:
        return (s, False)
    else:
        return ('Error in input', True)


def check(s):
    """Check if there is no duplicate numbers in same row, column and box."""
    b = box()
    for i in range(81):
        if s[i] > 0:
            for j in b[i]:
                if s[i] == s[j]:
                    return ('Both ' + cell(i) + ' and ' + cell(j)
                            + ' have the same value of ' + str(
                            s[i]) + '.', True)
    return ('', False)


def box():
    """Define the effective cells."""
    box = []
    for i in range(9):
        for j in range(9):
            b = [0] * 81
            for i2 in range(9):
                b[i2 * 9 + j] = 1
            for j2 in range(9):
                b[i * 9 + j2] = 1
            begin = (i // 3) * 27 + (j // 3) * 3
            for i2 in [0, 1, 2]:
                for j2 in [0, 1, 2]:
                    b[begin + i2 * 9 + j2] = 1
            b[i * 9 + j] = 0
            bb = []
            for i2 in range(81):
                if b[i2] == 1:
                    bb = bb + [i2]
            box = box + [bb]
    return (box)


def pbox():
    """List for use in calculating pointing pair and triple."""
    pbox = {}
    for b in range(9):
        begin = [0, 3, 6, 27, 30, 33, 54, 57, 60][b]
        left = [0, 0, 0, 27, 27, 27, 54, 54, 54][b]
        top = [0, 3, 6, 0, 3, 6, 0, 3, 6][b]
        box = set([])
        for i in (0, 1, 2, 9, 10, 11, 18, 19, 20):
            box = box | set([begin + i])
        for j in range(3):
            line = set([])
            for i in range(9):
                line = line | set([left + j * 9 + i])
            pbox[(b, j)] = line - box
        for j in range(3, 6):
            line = set([])
            for i in range(9):
                line = line | set([top + j - 3 + i * 9])
            pbox[(b, j)] = line - box
    return (pbox)


def boxlist(s):
    """Boxes, row, and columns to scan."""
    boxlist = []
    for b in range(9):
        begin = [0, 3, 6, 27, 30, 33, 54, 57, 60][b]
        bo = []
        for i in (0, 1, 2, 9, 10, 11, 18, 19, 20):
            if s[begin + i] == 0:
                bo.append(begin + i)
        boxlist.append(bo)
    for i in range(9):
        line = []
        for j in range(9):
            if s[i * 9 + j] == 0:
                line.append(i * 9 + j)
        boxlist.append(line)
    for j in range(9):
        line = []
        for i in range(9):
            if s[i * 9 + j] == 0:
                line.append(i * 9 + j)
        boxlist.append(line)

    return boxlist


def combmir(p, boxl):
    """Make lists for calculating naked and hidden combination."""
    comb = []
    mirror = []
    for bo in boxl:
        co = []
        mi = []
        for i in bo:
            c = ()
            for j in range(9):
                if p[i][j] == 1:
                    c = c + (j + 1,)
            co.append(c)
        comb.append(co)
        n = []
        for i in co:
            for j in i:
                if j not in n:
                    n.append(j)
        n = sorted(n)
        for i in n:
            m = ()
            for j in range(len(co)):
                if i in co[j]:
                    m = m + (j,)
            mi.append(m)
        mirror.append([n, mi])
    return (comb, mirror)


def pairs(s, p):
    """Extract pair."""
    pair = []
    paircomb = []
    for i in range(81):
        if s[i] == 0:
            if sum(p[i]) == 2:
                m = p[i].index(1) + 1
                n = p[i][m:].index(1) + m + 1
                pair.append([i, m, n])
                paircomb.append([i, m])
                paircomb.append([i, n])
    return (pair, paircomb)


def cell(i):
    """Address of the cell."""
    return ('R' + str(i // 9 + 1) + 'C' + str(i % 9 + 1))


def blank(s):
    """Count numbers of blank cells."""
    blank = 0
    for i in range(81):
        if s[i] == 0:
            blank = blank + 1
    return (blank)


def lev(i):
    """Description of levels."""
    return (['undefined', 'trivial', 'very easy', 'easy', 'normal', 'hard',
             'very hard', 'evil', 'extreme', 'ultimate'][i])


def duplicate(s1, s2):
    """Find different cells."""
    s = []
    for i in range(81):
        if s1[i] == s2[i]:
            s.append(s1[i])
        else:
            s.append(0)
    return (s)


def status(level, solved, err):
    """Calculate status from status and err."""
    if solved:
        if err:
            status = 'multiple solution'
        else:
            status = 'level ' + str(level)
    else:
        if err:
            status = 'no solution'
        else:
            status = 'give up'
    return status


def current(s, move):
    """Move to current position."""
    pos = 0
    for m in move:
        m = int(m)
        row = m // 100
        column = (int(m) - row * 100) // 10
        n = m % 10
        i = (row - 1) * 9 + column - 1
        if s[i] > 0:
            message = 'Cell ' + \
                cell(i) + ' is already filled with ' + str(s[i])
            if pos == 0:
                return (s, [], message, True)
            else:
                return (s, move[:pos], message, True)
        s2 = copy.copy(s)
        s2[i] = n
        message, err = check(s2)
        if err:
            if pos == 0:
                return (s, [], message, True)
            else:
                return (s, move[:pos], message, True)
        message = 'Moved: ' + cell(i) + ' = ' + str(n)
        s = s2
        pos += 1
    return s, move, message, False


def openappend(file):
    """Open as append if exists, write if not."""
    try:
        output = open(file, 'a')
    except Exception:
        try:
            output = open(file, 'w')
        except Exception:
            return 'error'
    return output



# -*- coding: utf-8 -*-
"""Calculation modules.

Modules for solving sudoku problems.
"""
import copy
import datetime
import itertools
from kaidoku.misc import blank
from kaidoku.misc import cell
from kaidoku.misc import check
from kaidoku.output import output


def solve(s, verbose, maxdepth, maxtime):
    """Solve a problem."""
    from kaidoku.misc import box
    from kaidoku.misc import pbox
    solved = False
    p = possible(s)
    b = box()
    pb = pbox()
    start = datetime.datetime.now()
    endtime = start + datetime.timedelta(seconds=maxtime)
    LevelPoint = int((blank(s) / 15.0)**3)  # 30 -> 8, 64 -> 77
    while (not solved):
        bl = blank(s)
        s, p, message, logic, depth, found, err = solveone(
            s, p, verbose, 0, maxdepth, endtime, b, pb)
        if err:
            if found:
                return (s, message, 0, True, True)
            else:
                return (s, message, 0, False, True)
        if found:
            if verbose == 4:
                message = str(81 - bl) + ': ' + message
            if message != '':
                print(message)
            if verbose > 4 and blank(s) < bl:
                print(str(81 - blank(s)) + ' numbers determined')
                print(output(s))
            LevelPoint += {
                'Naked single': 0,
                'Hidden single':  3,
                'Pointing pair': 25,
                'Pointing triple': 25,
                'Naked pair': 25,
                'Naked triple': 40,
                'Hidden pair': 50,
                'Hidden triple': 70,
                'X-wing': 70,
                'XY-wing': 70,
                'XYZ-wing': 100,  # not implemented yet
                'Swordfish': 100,  # not implemented, rare
                'Jellyfish': 100,  # not implemented, rare
                "Bowman's Bingo": 100,  # not sure
                'Naked quad': 100,  # not implemented yet
                'Hidden quad': 100,  # not implemented yet
                'Remote pairs': 100,  # not implemented yet
                'Chain of pairs': 200,
                'Chain of pairs (long)': 300,
                'Trial': 500,
                'Trial (deep)': 1000,
                'Search': 1500,
                'Deep search': 2500
            }[logic]
        else:
            return (s, 'Give up', 0, False, False)
        if blank(s) == 0:
            solved = True
        else:
            solved = False
    for i in range(9):
        if [0, 8, 45, 70, 120, 320, 1000, 2000, 5500][i] <= LevelPoint:
            level = i + 1
    return (s, 'Solved', level, True, False)


def solveone(s, p, verbose, depth, maxdepth, endtime, b, pb):
    """Solve one move."""
    from kaidoku.chain import pairchain
    from kaidoku.misc import boxlist
    from kaidoku.misc import combmir
    from kaidoku.misc import pairs
    from kaidoku.search import search
    from kaidoku.search import trial
    from kaidoku.wing import xwing
    from kaidoku.wing import xywing
    # Check validity
    message, err = check(s)
    if verbose == 0:
        message = ''
    if err:
        return (s, p, message, 0, depth, False, True)

    # Basic logic
    boxl = boxlist(s)
    s, p, message, found, err = naksing(s, p, b, verbose)  # Naked single
    if found or err:
        return (s, p, message, 'Naked single', depth, found, err)

    # Intermediate logic
    comb, mirror = combmir(p, boxl)
    if depth < 2:
        s, p, message, found, err = hidsing(s, p, verbose)  # Hidden single
        if found or err:
            return (s, p, message, 'Hidden single', depth, found, err)
        s, p, logic, message, found, err = pointing(
            s, p, pb, verbose)  # Pointing pair or triple
        if found or err:
            return (s, p, message, logic, depth, found, err)
        if depth < 1:
            onlypair = False
        else:
            onlypair = True
        s, p, logic, message, found, err = nakhid(
            s, p, boxl, comb, mirror, verbose, onlypair)  # Naked or hidden
        if found or err:
            return (s, p, message, logic, depth, found, err)

    # Advanced logic
    pair, paircomb = pairs(s, p)
    if depth == 0:
        s, p, message, found, err = xwing(
            s, p, b, boxl, mirror, verbose)  # X-wing
        if found or err:
            return (s, p, message, 'X-wing', depth, found, err)
        s, p, message, found, err = xywing(s, p, b, pair, verbose)  # XY-wing
        if found or err:
            return (s, p, message, 'XY-wing', depth, found, err)

        if len(pair) > 3:
            s, p, message, chainlength, found, err = pairchain(
                s, p, b, pair, paircomb, verbose)  # Chain of pairs
            if chainlength < 8:
                logic = 'Chain of pairs'  # Short chain
            else:
                logic = 'Chain of pairs (long)'  # Long chain
            if found or err:
                return (s, p, message, logic, depth, found, err)

    # Trial
    # Maximum steps of scan distinguished from full search of 1 depth
    maxstep = 15
    if depth == 0 and maxdepth < 999:
        s, p, step, message, found, err = trial(
            s, p, maxstep, verbose, b, pair)
        if step < 10:
            logic = 'Trial'
        else:
            logic = 'Trial (deep)'
        if found or err:
            return (s, p, message, logic, depth, found, err)

    # Search from minimum depth
    if maxdepth == 999:
        bla = blank(s)  # Creating mode
    else:
        bla = 0

    if depth == 0:
        if maxdepth > 2 and bla < 60:
            s, p, message, logic, depth2, found, err = search(
                s, p, verbose, 1, 2, endtime, b, pb)
            if found or err:
                return (s, p, message, logic, depth2, found, err)
        if maxdepth > 3 and bla < 45:
            s, p, message, logic, depth2, found, err = search(
                s, p, verbose, 1, 3, endtime, b, pb)
            if found or err:
                return (s, p, message, logic, depth2, found, err)
        if maxdepth > 4 and bla < 35:
            s, p, message, logic, depth2, found, err = search(
                s, p, verbose, 1, 4, endtime, b, pb)
            if found or err:
                return (s, p, message, logic, depth2, found, err)

    # Full search
    if maxdepth > depth:
        s, p, message, logic, depth, found, err = search(
            s, p, verbose, depth + 1, maxdepth, endtime, b, pb)
        if found or err:
            return (s, p, message, logic, depth, found, err)

    return (s, p, '', 0, depth, False, False)


def possible(s):
    """Make possibility array."""
    from kaidoku.misc import box
    p = [[]] * 81
    b = box()
    for i in range(81):
        if s[i] == 0:
            p[i] = [1] * 9
            for i2 in b[i]:
                if s[i2] > 0:
                    p[i][s[i2] - 1] = 0
        else:
            p[i] = [0] * 9
            p[i][s[i] - 1] = 1
    return (p)


def naksing(s, p, b, verbose):
    """Naked single."""
    for i in range(81):
        if s[i] == 0:
            for i2 in b[i]:
                v = s[i2]
                if v > 0:
                    p[i][v - 1] = 0
        else:
            p[i] = [10]

    found = False
    message = ''
    for i in range(81):
        if (sum(p[i])) == 0:
            if verbose > 0:
                message = 'This sudoku has no solution because ' + \
                    cell(i) + ' has no candidate.'
            return (s, p, message, False, True)
        if (sum(p[i])) == 1:
            i2 = p[i].index(1)
            found = True
            s[i] = i2 + 1
            if verbose > 3:
                if message == '':
                    message = ('Naked single: ')
                else:
                    message = message + ", "
                message = message + cell(i) + " = " + str(i2 + 1)

    if blank(s) == 0:
        message2, err = check(s)
        if verbose == 0:
            message2 = ''
        if err:
            return (s, p, message2, False, True)

    return (s, p, message, found, False)


def hidsing(s, p, verbose):
    """Hidden single."""
    for b in range(9):
        begin = [0, 3, 6, 27, 30, 33, 54, 57, 60][b]
        for n in range(9):
            sum = 0
            for i in (0, 1, 2, 9, 10, 11, 18, 19, 20):
                if s[begin + i] == 0:
                    sum = sum + p[begin + i][n]
            if sum == 1:
                for i in (0, 1, 2, 9, 10, 11, 18, 19, 20):
                    if (s[begin + i] == 0) and (p[begin + i][n]) == 1:
                        s[begin + i] = n + 1
                        if verbose < 4:
                            message = ''
                        else:
                            message = 'Hidden single in box ' + \
                                str(b + 1) + ' : ' + \
                                cell(begin + i) + ' = ' + str(n + 1)
                        return (s, p, message, True, False)

    for i in range(9):
        for n in range(9):
            sum = 0
            for j in range(9):
                if s[i * 9 + j] == 0:
                    sum = sum + p[i * 9 + j][n]
            if sum == 1:
                for j in range(9):
                    if (s[i * 9 + j] == 0) and (p[i * 9 + j][n]) == 1:
                        s[i * 9 + j] = n + 1
                        if verbose < 4:
                            message = ''
                        else:
                            message = 'Hidden single in row ' + \
                                str(i + 1) + ' : ' + \
                                cell(i * 9 + j) + ' = ' + str(n + 1)
                        return (s, p, message, True, False)

    for j in range(9):
        for n in range(9):
            sum = 0
            for i in range(9):
                if s[i * 9 + j] == 0:
                    sum = sum + p[i * 9 + j][n]
            if sum == 1:
                for i in range(9):
                    if (s[i * 9 + j] == 0) and (p[i * 9 + j][n]) == 1:
                        s[i * 9 + j] = n + 1
                        if verbose < 4:
                            message = ''
                        else:
                            message = 'Hidden single in column ' + \
                                str(j + 1) + ' : ' + \
                                cell(i * 9 + j) + ' = ' + str(n + 1)
                        return (s, p, message, True, False)

    return (s, p, '', False, False)


def pointing(s, p, pb, verbose):
    """Pointing pair or triple."""
    for b in range(9):
        begin = [0, 3, 6, 27, 30, 33, 54, 57, 60][b]
        blank = set()
        for i in range(9):
            if s[begin + (0, 1, 2, 9, 10, 11, 18, 19, 20)[i]] == 0:
                blank.add(i)
        for i in range(6):
            j = [set([0, 1, 2]), set([3, 4, 5]), set([6, 7, 8]),
                 set([0, 3, 6]), set([1, 4, 7]), set([2, 5, 8])][i]
            b2 = blank & j
            if len(b2) > 1 > 0:
                for n in range(9):
                    sum = 0
                    for a in b2:
                        sum = sum + \
                            p[begin + (0, 1, 2, 9, 10, 11, 18, 19, 20)[a]][n]
                    if sum > 1:
                        sum2 = 0
                        sum3 = 0
                        for a in (blank - b2):
                            sum2 = sum2 + \
                                p[begin + (0, 1, 2, 9, 10, 11,
                                           18, 19, 20)[a]][n]
                        for a in pb[(b, i)]:
                            if s[a] == 0:
                                sum3 = sum3 + p[a][n]
                        if (sum2 == 0) != (sum3 == 0):
                            logic = 'Pointing ' + \
                                ['', '', 'pair', 'triple'][sum]
                            message = ''
                            if verbose > 2:
                                message = logic + ' in box ' + str(b + 1)
                            if verbose > 3:
                                message = message + " removed " + \
                                    str(n + 1) + " from "
                            if (sum2 == 0):
                                for a in pb[(b, i)]:
                                    if s[a] == 0 and p[a][n] == 1:
                                        p[a][n] = 0
                                        if verbose > 3:
                                            message = message + cell(a) + ' '
                                        if p[a].count(1) == 1:
                                            if verbose > 3:
                                                message = message + \
                                                    '(=' + \
                                                    str(p[a].index(
                                                        1) + 1) + ') '
                                            s[a] = p[a].index(1) + 1
                            else:
                                for a in (blank - b2):
                                    a2 = begin + (0, 1, 2, 9, 10,
                                                  11, 18, 19, 20)[a]
                                    if p[a2][n] == 1:
                                        r = p[a2]
                                        r[n] = 0
                                        p[a2] = r
                                        if verbose > 3:
                                            message = message + cell(a2) + ' '
                                        if p[a2].count(1) == 1:
                                            if verbose > 3:
                                                message = message + \
                                                    '(=' + \
                                                    str(p[a2].index(
                                                        1) + 1) + ') '
                                            s[a2] = p[a2].index(1) + 1
                            return (s, p, logic, message, True, False)

    return (s, p, '', '', False, False)


def nakhid(s, p, boxl, comb, mirror, verbose, onlypair):
    """Naked and hidden pair or triple."""
    # Naked pair
    for i in range(len(comb)):
        list = comb[i]
        if len(list) > 3:
            np, pa, position, f = nakpair(list)
            if np:
                message = ''
                if verbose > 2:
                    message = 'Naked pair in ' + \
                        ['box ', 'row ', 'column '][i // 9] + str(i % 9 + 1)
                if verbose > 3:
                    message = message + ' made removal from'
                for j in f:
                    k = boxl[i][j]
                    p[k][pa[0] - 1] = 0
                    p[k][pa[1] - 1] = 0
                    if verbose > 3:
                        message = message + ' ' + cell(k)
                    if p[k].count(1) == 1:
                        if verbose > 3:
                            message = message + \
                                '(=' + str(p[k].index(1) + 1) + ') '
                        s[k] = p[k].index(1) + 1
                return (s, p, 'Naked pair', message, True, False)

    # Naked triple
    if not onlypair:
        for i in range(len(comb)):
            list = comb[i]
            if len(list) > 5:
                nt, tri, position, f = naktriple(list)
                if nt:
                    message = ''
                    if verbose > 2:
                        message = 'Naked triple in ' + \
                            ['box ', 'row ', 'column '][i // 9] + \
                            str(i % 9 + 1)
                    if verbose > 3:
                        message = message + ' made removal from'
                    for j in f:
                        k = boxl[i][j]
                        p[k][tri[0] - 1] = 0
                        p[k][tri[1] - 1] = 0
                        p[k][tri[2] - 1] = 0
                        if verbose > 3:
                            message = message + ' ' + cell(k)
                        if p[k].count(1) == 1:
                            if verbose > 3:
                                message = message + \
                                    '(=' + str(p[k].index(1) + 1) + ') '
                            s[k] = p[k].index(1) + 1
                    return (s, p, 'Naked triple', message, True, False)

    # Hidden pair
    for i in range(len(mirror)):
        list2 = mirror[i][1]
        if len(list2) > 3:
            hp, pa, position, f = nakpair(list2)
            if hp:
                p1 = mirror[i][0][position[0]]
                p2 = mirror[i][0][position[1]]
                message = 'Hidden pair of (' + str(p1) + ', ' + str(
                    p2) + ') in ' + ['box ', 'row ', 'column '][i // 9] + str(
                    i % 9 + 1) + ': ' + cell(boxl[i][pa[0]]) + ' ' + cell(
                    boxl[i][pa[1]])
                r = [0] * 9
                r[p1 - 1] = 1
                r[p2 - 1] = 1
                p[boxl[i][pa[0]]] = r
                p[boxl[i][pa[1]]] = copy.copy(r)
                message = ''
                if verbose > 2:
                    message = 'Hidden pair in ' + \
                        ['box ', 'row ', 'column '][i // 9] + str(i % 9 + 1)
                return (s, p, 'Hidden pair', message, True, False)

    # Hidden triple
    if not onlypair:
        for i in range(len(mirror)):
            list2 = mirror[i][1]
            if len(list2) > 3:
                ht, tri, position, f = naktriple(list2)
                if ht:
                    rem = set([1, 2, 3, 4, 5, 6, 7, 8, 9])
                    for k in range(3):
                        rem.remove(mirror[i][0][position[k]])
                    message = 'Hidden triple of ' + str(tri) + ' in ' + [
                        'box ', 'row ', 'column '][i // 9] + str(
                        i % 9 + 1) + ': ' + cell(boxl[i][tri[0]]) + ' ' + cell(
                        boxl[i][tri[1]]) + ' ' + cell(boxl[i][tri[2]])
                    for k in range(3):
                        for m in rem:
                            p[boxl[i][tri[k]]][m - 1] = 0

                    message = ''
                    if verbose > 2:
                        message = 'Hidden triple in ' + \
                            ['box ', 'row ', 'column '][i // 9] + \
                            str(i % 9 + 1)
                    return (s, p, 'Hidden triple', message, True, False)

    return (s, p, 5, '', False, False)


def nakpair(list):
    """Naked pair."""
    pair = []
    for i in list:
        if len(i) == 2:
            pair.append(i)
    if len(set(pair)) < len(pair):
        pairs = []
        for i in range(len(pair)):
            for j in range(i + 1, len(pair)):
                if pair[i] == pair[j]:
                    pairs.append(pair[i])
        for pa in pairs:
            other = set(list) - set([pa])
            f = []
            for q in other:
                if pa[0] in q or pa[1] in q:
                    f.append(list.index(q))
            if len(f) > 0:
                position = [i for i, x in enumerate(list) if x == pa]
                return (True, pa, position, f)

    return (False, 0, 0, 0)


def naktriple(list):
    """Naked triple."""
    triple = []
    num = set()
    for i in list:
        if len(i) < 4:
            triple.append(i)
            for j in i:
                num.add(j)
        for c in itertools.combinations(num, 3):
            n = 0
            for k in triple:
                if set(k).issubset(set(c)):
                    n += 1
            if n == 3:
                tri = []
                position = []
                m = 0
                f = []
                for k in list:
                    if set(k).issubset(set(c)):
                        tri.append(k)
                        position.append(m)
                    else:
                        if set(k).intersection(set(c)):
                            f.append(m)
                    m += 1
                if len(f) > 0:
                    return (True, c, position, f)

    return (False, 0, 0, 0)



# -*- coding: utf-8 -*-
"""Chain logic."""


def pairchain(s, p, b, pair, paircomb, verbose):
    """Chain of pairs."""
    from kaidoku.misc import cell
    message = ''
    if len(pair) < 4:
        return (s, p, message, False, False)
    for i in paircomb:
        chain = [i]
        chaincell = [i[0]]
        chain2 = findchain(pair, i, b)
        key = i[0] * 10 + i[1]
        dict = {key: chain2}
        path = {key: [i]}
        if chain2 != []:
            for j in chain2:
                chaincell.append(j[0])
            for j in chain2:
                chain.append(j)
                path[j[0] * 10 + j[1]] = path[i[0] * 10 + i[1]] + [j]
            found = True
            loop = False
            while found:
                found = False
                for j in chain2:
                    jkey = j[0] * 10 + j[1]
                    if jkey in dict:
                        chain3 = dict[jkey]
                    else:
                        chain3 = findchain(pair, j, b)
                        dict[jkey] = chain3
                    for j3 in chain3:
                        path[j3[0] * 10 + j3[1]] = path[jkey] + [j3]
                    if chain3 != []:
                        for j2 in chain3:
                            if j2[0] in chaincell:
                                for k in chain:
                                    if k[0] == j2[0]:
                                        if k[1] == j2[1]:
                                            loop = True
                                        else:
                                            # Found a chain
                                            path1 = path[j2[0] * 10 + j2[1]]
                                            path2 = path[j2[0] * 10 + k[1]]
                                            for i in path1[1:]:
                                                if i in path2:
                                                    p1 = path1.index(i)
                                                    path1 = path1[p1:]
                                                    p2 = path2.index(i)
                                                    path2 = path2[p2:]
                                            i = path1[0]
                                            p[i[0]][i[1] - 1] = 0
                                            n = p[i[0]].index(1) + 1
                                            s[i[0]] = n
                                            chainlength = len(
                                                path1) + len(path2)
                                            if verbose > 1:
                                                if verbose == 2:
                                                    message = 'Chain of pairs. Start from ' + \
                                                        cell(i[0]) + '.'
                                                else:
                                                    message = 'Chain of pairs. Assume that ' + \
                                                        cell(i[0]) + ' is ' + str(i[1]) + \
                                                        ' and we have following chains.\n'
                                                    message += '(1) ' + chainpath(path1) + \
                                                        '\n' + \
                                                        '(2) ' + \
                                                        chainpath(path2) + '\n'
                                                    message += 'Now we have contradiction on ' + \
                                                        cell(j2[0]) + '. Therefore ' + cell(i[0]) + \
                                                        ' should be ' + \
                                                        str(n) + '. '
                                            return (s, p, message, chainlength, True, False)
                            chaincell.append(j2[0])
                        if not loop:
                            chain2 = chain3
                            for j in chain2:
                                chain.append(j)
                            found = True

    return (s, p, message, 0, False, False)


def findchain(pair, i, b):
    """Find chain."""
    chain = []
    for j in pair:
        if j[0] in b[i[0]] and (i[1] == j[1] or i[1] == j[2]):
            if i[1] == j[1]:
                chain.append([j[0], j[2]])
            else:
                chain.append([j[0], j[1]])
    return chain


def chainpath(path):
    """Get path of chain."""
    from kaidoku.misc import cell
    chainpath = ''
    for i in path:
        if i != path[0]:
            chainpath += ' >> '
        chainpath += cell(i[0]) + ' = ' + str(i[1])
    return chainpath



# -*- coding: utf-8 -*-
"""Help messages."""


def welcommessage(version):
    """Return welcome message."""
    message = 'Kaidoku - player, solver and creater of sudoku puzzles.\n' + \
        'Type h for help, c for showing a problem, q for quit.'
    return message


def helpmessage():
    """Return help message."""
    message = '246 : In the cell of row 2 column 4, put number 6\n'
    message += 'b   : take Back one move\n'
    message += 'c   : show Current position\n'
    message += 'i   : show hInt for current position\n'
    message += 'q   : Quit kaidoku\n'
    message += 'u   : show URL of the current position\n\n'
    message += '[Select problem]\n'
    message += 'j num  : Jump to problem num\n'
    message += 'n      : Next problem\n'
    message += 'p      : Previous problem\n'
    message += 'l level: change Level\n'
    message += 'book   : show index of the book\n\n'
    message += '[Help]\n'
    message += 'h  : show this Help\n'
    message += 'ha : show Help for Advanced commands'
    return message


def advancedhelp():
    """Return help message for advanced commands."""
    message = 'a [verb]        : Analyze the current problem\n'
    message += 'ac [verb]       : Analyze from the Current position\n'
    message += 'all [verb]      : analyze all the problems in the current level\n'
    message += 'append pos      : append a position to book\n'
    message += 'ba [label]      : add current position to bookmark\n'
    message += 'bl              : list bookmark\n'
    message += 'bp pos          : add a specified position bookmark\n'
    message += 'br label        : read bookmark\n'
    message += 'config          : show configuration\n'
    message += 'create [numbers]: create new problems and append to book\n'
    message += 'giveup [sec]    : reanalyze giveup positions\n'
    message += 'jpg             : draw jpeg image\n'
    message += 'jm              : draw jpeg image with mark\n'
    message += 'import          : import file2\n'
    message += 'initial         : go back to initial position\n'
    message += 'maxtime [sec]   : set maximum time for analyzing a position (todo)\n'
    message += 'reanalyze       : reanalyze whole book\n'
    message += 'solve pos [verb]: solve a specified position\n'
    message += 'sp              : Solve Partially with naked or hidden single'
    return message
